clear all
randn('seed',0);
rand('seed',0);
%build the graphs of f and F, with F given at different values of T

w=-2.5:0.001:2.5; 
f1=-4*w.^2-w.^3+w.^4;
[Y,I]=min(f1);
fprintf('w opt %f\n',w(I))


t=0.01;
g=exp(-w.^2/2/t);
F1=-log(conv(exp(-f1),g,'same'));
[Y1,I1]=min(F1);
fprintf('w opt for F1 %f\n',w(I1))
t=0.8;
g=exp(-w.^2/2/t);
F2=-log(conv(exp(-f1),g,'same'));
[Y3,I3]=min(F2);
fprintf('w opt for F2 %f\n',w(I3))


figure(1)
plot(w,f1,w,F1,w,F2);
legend('f','F (T=0.01)','F (T=0.8)')
xlabel('w')
print -depsc adsamp1
%now compute w* as a gradient descent, but use the values of u as gradient
figure(2)
ntries=21;
eta=0.05;
csi=0.05;
nu=1;
n_traj= 50;                 %number of trajectories
l = 100;                     %max number of updates of the parameters A(t) and B(t)
w_desc=zeros(ntries,l);
S_all=zeros(l,n_traj);      %stores the exponent of the weight, for each cycle
alpha = zeros(l,n_traj);    %stores the weights
u=0;
ess_all= zeros(l,1);        %stores the number of samples reaching the target, for each cycle
x_all(:,1)=2*ones(n_traj,1);
N=l;                      %sets the number of timesteps in the discretization
a=zeros(N,1); 
b=zeros(N,1);
for j=1:ntries
	w=-1+0.1*(j-1);
	i=0;
    a=zeros(N,1); 
    b=zeros(N,1);
    T=0.8;
    dt=T/(N);
    eta=0.05;
	for k=1:l 
        x=zeros(1,n_traj)+ w;
        noise = zeros(n_traj,N);        %stores all noise during a trajectory, for all trajectories
        x_all=zeros(n_traj,N+1);    %stores all the trajectory point given a value of u, for each trajectory 
        uall =zeros(n_traj,N);
        for i=1:N                                   %creates n_traj trajectories at the same time         
            dw = sqrt(dt*nu)*randn(1,n_traj);
            x = x + (a(i)*x+b(i))*dt + dw;
            uall(:,i)=(a(i)*x+b(i));
            x_all(:,i+1)=x;
            noise(:,i)=dw;
        end;
        xall = x_all(:,1:N);
        wx_all= xall.*noise;
        f1=-4*x_all(:,N+1).^2-x_all(:,N+1).^3+x_all(:,N+1).^4;
        S_all(k,:) =f1 + 0.5*dt*sum(uall.^2,2)+sum(noise.*uall,2);
        S_all(k,:) = S_all(k,:)- min(S_all(k,:));
        alpha(k,:)=exp(-S_all(k,:)/nu);
        psi=mean(alpha(k,:));
        J_all(j,k)=-nu*log(psi*n_traj);
        beta=alpha(k,:)/(n_traj*psi);
        ess=-1/log(n_traj)*sum(beta.*log(beta));
        ess_all(k)=ess;
        %update of a(t) and b(t)
        for i=2:N-1
            r = sum(noise(:,i:(i+1)),2);
            q = sum(wx_all(:,i:(i+1)),2);
            Q1 = mean(r'.*alpha(k,:))/(psi);
            Q2 = mean(q'.*alpha(k,:))/(psi);
            H1 =mean(xall(:,i)'.*alpha(k,:))/psi;
            H2 = mean((xall(:,i).^2)'.*alpha(k,:))/psi;
            a(i)=a(i)+csi*1/(2*dt)*1/(H2-H1^2)*(Q2 - Q1*H1);
            b(i)=b(i)+csi*1/(2*dt)*1/(H2-H1^2)*(H2*Q1 -H1*Q2);
        end;
        %a(N-1)=a(N-5);
        a(N)=a(N-1);
        %a(N-1)=a(N-5);
        %a(N-2)=a(N-5);
        %a(N-3)=a(N-5);
        %a(N-4)=a(N-5);
        %b(N-1)=b(N-5);
        b(N)=b(N-1);
        %b(N-2)=b(N-5);
        %b(N-3)=b(N-5);
        %b(N-4)==b(N-5);
        for i=2:N-1
            a(i)=(a(i-1)+a(i)+a(i+1))/3;
            b(i)=(b(i-1)+b(i)+b(i+1))/3;
        end;
        a(1)=a(3);
        %a(2)=a(3);
        a(N)=a(N-1);
        %a(N-1)=a(N-2);
        b(1)=b(3);
        %b(2)=b(3);
        b(N)=b(N-1);
        %b(N-1)=b(N-2);
        %T=0.001+1/(1+exp(i-l*0.7));
        %eta=0.1/T;
        u = a(l)*w+b(l);
        w = w+eta*u*T;
        w_desc(j,k)=w;
    end;

end;
mean(w_desc(:,l))
clf
%subplot(2,2,1)
plot(w_desc')
xlabel('iteration');
ylabel('w')
print -depsc adsamp2

%subplot(2,2,3)
%plot(f_all')
%xlabel('iteration');
%ylabel('f')
%wfinal=w_all(:,maxi);
%subplot(2,2,4)
%plot(J_all')
%xlabel('iteration');
%ylabel('F')
%mean(wfinal)
%std(wfinal)
%ffinal=f(wfinal,b,phi);
%mean(ffinal)
%std(ffinal)






